#!/bin/bash
set -x
#this scripts needs the following env variables to be set-up:
# - ppa_target
# - packaging_branch
# - merge_revision
# - trunk
# - DEBFULLNAME
# - DEBEMAIL
# - [distributions] (optional, comma separated list of target distributions)

#if we have a ppa, then do a dput
if [  -n "$ppa_target"  -a  -n "$packaging_branch" -a \
      -n "$merge_revision" -a -n "$trunk" -a -n "$DEBFULLNAME" -a \
      -n "$DEBEMAIL" ]; then

    TMPFILE=`mktemp -d`
    cd $TMPFILE
    bzr checkout -r $merge_revision $trunk trunk
    cd trunk
    #first we need to merge the packaging branch and create a source package
    bzr merge "$packaging_branch" || result=1

    # bump the revision to next packaging version and create a source tarball
    version=`dpkg-parsechangelog | awk '/^Version/ {print $2}' | sed -e "s/\(.*\)-[0-9]ubuntu.*/\1/"`+bzr${merge_revision}
    version=${version}ubuntu0 #+${packaging_rev}
    sourcename=`dpkg-parsechangelog | awk '/^Source/ {print $2}'`
    
    if [ -z "$distributions" ]; then 
        dch -v $version "Automatic build of revision $merge_revision"
        cd ..
        tar -czf ${sourcename}_${version}.orig.tar.gz trunk
        cd trunk

        debuild -S -d || result=1
        cd ..
        dput "$ppa_target" *_source.changes || result=1
        cd trunk
    else
        cp debian/changelog ../changelog-copy
        echo "$distributions," | while read -d, distribution; do
            echo "Going to dput into '$distribution'"
            dch -v $version~${distribution}1 \
                "Automatic build of revision $merge_revision" -D \
                "$distribution"
            cd ..
            tar -czf ${sourcename}_${version}~${distribution}1.orig.tar.gz trunk
            cd trunk
    
            debuild -S -d || result=1
            cd ..
            dput "$ppa_target" *${distribution}1_source.changes || result=1
            cd trunk
            cp ../changelog-copy debian/changelog
        done
    fi

    cd ..
    rm -rf $TMPFILE
    exit 0
else
    echo 'One of $ppa_target, $packaging_branch, $trunk, $merge_revision, $DEBFULLNAME, $DEBEMAIL is missing'
    exit 1    
fi
