#!/usr/bin/env python

import sys
from argparse import ArgumentParser
from launchpad import LaunchpadTrigger


def main():
    parser = ArgumentParser(description="Vote on a Launchpad merge proposal.")
    parser.add_argument('-s', '--status')
    parser.add_argument('-u', '--build-url', required=True,
                        help="URL of the Jenkins job")
    parser.add_argument('-r', '--revision', required=True,
                        help="merge proposal candidate revision")
    parser.add_argument('-b', '--branch',
                        help="Launchpad branch proposed for merging")
    parser.add_argument('-p', '--merge-proposal', required=True,
                        help="URL of the merge proposal to update")
    args = vars(parser.parse_args())
    launchpad = LaunchpadTrigger()
    if not args['branch']: 
        # if branch was not specified try to get it from merge proposal
        args['branch'] = launchpad.get_branch_from_mp(args['merge_proposal'])
        if not args['branch']:
            parser.error('Error: unable to get branch from merge proposal.\n'+
                         'Please specify manually using -- branch.')
    mp = launchpad.get_merge_proposal(args['branch'], args['merge_proposal'])
    if not mp:
        parser.error('merge proposal relted to this branch was not found')
    if args['status'] == 'PASSED': # status == True corresponds to 'PASSED'
        launchpad.approve_mp(mp, args['revision'], args['build_url'])
    else: # status == False corresponds to NOT 'PASSED'
        launchpad.disapprove_mp(mp, args['revision'], args['build_url'])
    sys.exit(0)


if __name__ == "__main__":
    sys.exit(main())
